@@ -12,13 +12,10 @@ use std::f32;
 use std::ops::Range;
 use std::rc::Rc;
 
-use crate::{ColorsLinear, Config, DimensionsParams, DimensionsWindow, Theme, Window};
+use crate::{dim, ColorsLinear, Config, Theme};
 use kas::cast::Cast;
 use kas::dir::{Direction, Directional};
-use kas::draw::{
-    self, color::Rgba, Draw, DrawRounded, DrawShared, ImageId, InputState, Pass, RegionClass,
-    SizeHandle, TextClass, ThemeApi,
-};
+use kas::draw::{self, color::Rgba, *};
 use kas::geom::*;
 use kas::text::format::FormattableText;
 use kas::text::{fonts, AccelString, Effect, Text, TextApi, TextDisplay};
@@ -36,6 +33,12 @@ pub struct FlatTheme {
     pub(crate) fonts: Option<Rc<LinearMap<TextClass, fonts::FontId>>>,
 }
 
+impl Default for FlatTheme {
+    fn default() -> Self {
+        Self::new()
+    }
+}
+
 impl FlatTheme {
     /// Construct
     #[inline]
@@ -69,7 +72,7 @@ impl FlatTheme {
     }
 }
 
-const DIMS: DimensionsParams = DimensionsParams {
+const DIMS: dim::Parameters = dim::Parameters {
     outer_margin: 8.0,
     inner_margin: 1.2,
     text_margin: 2.0,
@@ -80,26 +83,23 @@ const DIMS: DimensionsParams = DimensionsParams {
     progress_bar: Vec2::splat(12.0),
 };
 
-pub struct DrawHandle<'a, D: DrawShared> {
-    pub(crate) shared: &'a mut D,
-    pub(crate) draw: &'a mut D::Draw,
-    pub(crate) window: &'a mut DimensionsWindow,
+pub struct DrawHandle<'a, DS: DrawSharedImpl> {
+    pub(crate) draw: DrawIface<'a, DS>,
+    pub(crate) window: &'a mut dim::Window,
     pub(crate) cols: &'a ColorsLinear,
-    pub(crate) offset: Offset,
-    pub(crate) pass: Pass,
 }
 
-impl<D: DrawShared> Theme<D> for FlatTheme
+impl<DS: DrawSharedImpl> Theme<DS> for FlatTheme
 where
-    D::Draw: DrawRounded,
+    DS::Draw: DrawRoundedImpl,
 {
     type Config = Config;
-    type Window = DimensionsWindow;
+    type Window = dim::Window;
 
     #[cfg(not(feature = "gat"))]
-    type DrawHandle = DrawHandle<'static, D>;
+    type DrawHandle = DrawHandle<'static, DS>;
     #[cfg(feature = "gat")]
-    type DrawHandle<'a> = DrawHandle<'a, D>;
+    type DrawHandle<'a> = DrawHandle<'a, DS>;
 
     fn config(&self) -> std::borrow::Cow<Self::Config> {
         std::borrow::Cow::Borrowed(&self.config)
@@ -113,7 +113,7 @@ where
         action
     }
 
-    fn init(&mut self, _draw: &mut D) {
+    fn init(&mut self, _shared: &mut SharedState<DS>) {
         let fonts = fonts::fonts();
         if let Err(e) = fonts.select_default() {
             panic!("Error loading font: {}", e);
@@ -121,14 +121,14 @@ where
         self.fonts = Some(Rc::new(
             self.config
                 .iter_fonts()
-                .filter_map(|(c, s)| fonts.select_font(&s).ok().map(|id| (*c, id)))
+                .filter_map(|(c, s)| fonts.select_font(s).ok().map(|id| (*c, id)))
                 .collect(),
         ));
     }
 
     fn new_window(&self, dpi_factor: f32) -> Self::Window {
-        let fonts = self.fonts.as_ref().clone().unwrap().clone();
-        DimensionsWindow::new(DIMS, self.config.font_size(), dpi_factor, fonts)
+        let fonts = self.fonts.as_ref().unwrap().clone();
+        dim::Window::new(DIMS, self.config.font_size(), dpi_factor, fonts)
     }
 
     fn update_window(&self, window: &mut Self::Window, dpi_factor: f32) {
@@ -136,40 +136,37 @@ where
     }
 
     #[cfg(not(feature = "gat"))]
-    unsafe fn draw_handle<'a>(
-        &'a self,
-        shared: &'a mut D,
-        draw: &'a mut D::Draw,
-        window: &'a mut Self::Window,
+    unsafe fn draw_handle(
+        &self,
+        draw: DrawIface<DS>,
+        window: &mut Self::Window,
     ) -> Self::DrawHandle {
-        // We extend lifetimes (unsafe) due to the lack of associated type generics.
-        unsafe fn extend_lifetime<'b, T>(r: &'b mut T) -> &'static mut T {
+        unsafe fn extend_lifetime<'b, T: ?Sized>(r: &'b T) -> &'static T {
+            std::mem::transmute::<&'b T, &'static T>(r)
+        }
+        unsafe fn extend_lifetime_mut<'b, T: ?Sized>(r: &'b mut T) -> &'static mut T {
             std::mem::transmute::<&'b mut T, &'static mut T>(r)
         }
-
         DrawHandle {
-            shared: extend_lifetime(shared),
-            draw: extend_lifetime(draw),
-            window: extend_lifetime(window),
-            cols: std::mem::transmute::<&'a ColorsLinear, &'static ColorsLinear>(&self.cols),
-            offset: Offset::ZERO,
-            pass: Pass::new(0),
+            draw: DrawIface {
+                draw: extend_lifetime_mut(draw.draw),
+                shared: extend_lifetime_mut(draw.shared),
+                pass: draw.pass,
+            },
+            window: extend_lifetime_mut(window),
+            cols: extend_lifetime(&self.cols),
         }
     }
     #[cfg(feature = "gat")]
     fn draw_handle<'a>(
         &'a self,
-        shared: &'a mut D,
-        draw: &'a mut D::Draw,
+        draw: DrawIface<'a, DS>,
         window: &'a mut Self::Window,
     ) -> Self::DrawHandle<'a> {
         DrawHandle {
-            shared,
             draw,
             window,
             cols: &self.cols,
-            offset: Offset::ZERO,
-            pass: Pass::new(0),
         }
     }
 
@@ -203,9 +200,9 @@ impl ThemeApi for FlatTheme {
     }
 }
 
-impl<'a, D: DrawShared> DrawHandle<'a, D>
+impl<'a, DS: DrawSharedImpl> DrawHandle<'a, DS>
 where
-    D::Draw: DrawRounded,
+    DS::Draw: DrawRoundedImpl,
 {
     /// Draw an edit box with optional navigation highlight.
     /// Return the inner rect.
@@ -218,15 +215,15 @@ where
         let inner1 = outer.shrink(self.window.dims.frame as f32 * BG_SHRINK_FACTOR);
         let inner2 = outer.shrink(self.window.dims.frame as f32);
 
-        self.draw.rect(self.pass, inner1, bg_col);
+        self.draw.rect(inner1, bg_col);
 
         // We draw over the inner rect, taking advantage of the fact that
         // rounded frames get drawn after flat rects.
         self.draw
-            .rounded_frame(self.pass, outer, inner2, BG_SHRINK_FACTOR, self.cols.frame);
+            .rounded_frame(outer, inner2, BG_SHRINK_FACTOR, self.cols.frame);
 
         if let Some(col) = nav_col {
-            self.draw.rounded_frame(self.pass, inner1, inner2, 0.6, col);
+            self.draw.rounded_frame(inner1, inner2, 0.6, col);
         }
 
         inner2
@@ -234,115 +231,103 @@ where
 
     /// Draw a handle (for slider, scrollbar)
     fn draw_handle(&mut self, rect: Rect, state: InputState) {
-        let outer = Quad::from(rect + self.offset);
+        let outer = Quad::from(rect);
         let thickness = outer.size().min_comp() / 2.0;
         let inner = outer.shrink(thickness);
         let col = self.cols.scrollbar_state(state);
-        self.draw.rounded_frame(self.pass, outer, inner, 0.0, col);
+        self.draw.rounded_frame(outer, inner, 0.0, col);
 
         if let Some(col) = self.cols.nav_region(state) {
             let outer = outer.shrink(thickness / 4.0);
-            self.draw.rounded_frame(self.pass, outer, inner, 0.6, col);
+            self.draw.rounded_frame(outer, inner, 0.6, col);
         }
     }
 }
 
-impl<'a, D: DrawShared> draw::DrawHandle for DrawHandle<'a, D>
+impl<'a, DS: DrawSharedImpl> draw::DrawHandle for DrawHandle<'a, DS>
 where
-    D::Draw: DrawRounded,
+    DS::Draw: DrawRoundedImpl,
 {
-    fn size_handle_dyn(&mut self, f: &mut dyn FnMut(&mut dyn SizeHandle)) {
-        unsafe {
-            let mut size_handle = self.window.size_handle(self.shared);
-            f(&mut size_handle);
-        }
+    fn size_handle(&mut self) -> &mut dyn SizeHandle {
+        self.window
     }
 
-    fn draw_device(&mut self) -> (kas::draw::Pass, Offset, &mut dyn kas::draw::Draw) {
-        (self.pass, self.offset, self.draw)
+    fn draw_device<'b>(&'b mut self) -> &mut dyn Draw {
+        &mut self.draw
     }
 
-    fn add_clip_region(
+    fn new_pass(
         &mut self,
-        rect: Rect,
+        inner_rect: Rect,
         offset: Offset,
-        class: RegionClass,
+        class: PassType,
         f: &mut dyn FnMut(&mut dyn draw::DrawHandle),
     ) {
-        let inner_rect = rect + self.offset;
-        let outer_rect = match class {
-            RegionClass::ScrollRegion => inner_rect,
-            RegionClass::Overlay => inner_rect.expand(self.window.dims.frame),
-        };
-        let pass = self.draw.add_clip_region(self.pass, outer_rect, class);
+        let mut outer_rect = inner_rect;
+        if class == PassType::Overlay {
+            outer_rect = inner_rect.expand(self.window.dims.frame);
+        }
+        let mut draw = self.draw.new_pass(outer_rect, offset, class);
 
-        if class == RegionClass::Overlay {
-            let outer = Quad::from(outer_rect);
-            let inner = Quad::from(inner_rect);
-            self.draw
-                .rounded_frame(pass, outer, inner, BG_SHRINK_FACTOR, self.cols.frame);
+        if class == PassType::Overlay {
+            let outer = Quad::from(outer_rect + offset);
+            let inner = Quad::from(inner_rect + offset);
+            draw.rounded_frame(outer, inner, BG_SHRINK_FACTOR, self.cols.frame);
             let inner = outer.shrink(self.window.dims.frame as f32 * BG_SHRINK_FACTOR);
-            self.draw.rect(pass, inner, self.cols.background);
+            draw.rect(inner, self.cols.background);
         }
 
         let mut handle = DrawHandle {
-            shared: self.shared,
-            draw: self.draw,
             window: self.window,
+            draw,
             cols: self.cols,
-            offset: self.offset - offset,
-            pass,
         };
         f(&mut handle);
     }
 
-    fn target_rect(&self) -> Rect {
-        // Translate to local coordinates
-        self.draw.get_clip_rect(self.pass) - self.offset
+    fn get_clip_rect(&self) -> Rect {
+        self.draw.get_clip_rect()
     }
 
     fn outer_frame(&mut self, rect: Rect) {
-        let outer = Quad::from(rect + self.offset);
+        let outer = Quad::from(rect);
         let inner = outer.shrink(self.window.dims.frame as f32);
         self.draw
-            .rounded_frame(self.pass, outer, inner, BG_SHRINK_FACTOR, self.cols.frame);
+            .rounded_frame(outer, inner, BG_SHRINK_FACTOR, self.cols.frame);
     }
 
     fn separator(&mut self, rect: Rect) {
-        let outer = Quad::from(rect + self.offset);
+        let outer = Quad::from(rect);
         let inner = outer.shrink(outer.size().min_comp() / 2.0);
         self.draw
-            .rounded_frame(self.pass, outer, inner, BG_SHRINK_FACTOR, self.cols.frame);
+            .rounded_frame(outer, inner, BG_SHRINK_FACTOR, self.cols.frame);
     }
 
     fn nav_frame(&mut self, rect: Rect, state: InputState) {
         if let Some(col) = self.cols.nav_region(state) {
-            let outer = Quad::from(rect + self.offset);
+            let outer = Quad::from(rect);
             let inner = outer.shrink(self.window.dims.inner_margin as f32);
-            self.draw.rounded_frame(self.pass, outer, inner, 0.0, col);
+            self.draw.rounded_frame(outer, inner, 0.0, col);
         }
     }
 
     fn selection_box(&mut self, rect: Rect) {
-        let inner = Quad::from(rect + self.offset);
+        let inner = Quad::from(rect);
         let outer = inner.grow(self.window.dims.inner_margin.into());
         // TODO: this should use its own colour and a stippled pattern
         let col = self.cols.text_sel_bg;
-        self.draw.frame(self.pass, outer, inner, col);
+        self.draw.frame(outer, inner, col);
     }
 
     fn text(&mut self, pos: Coord, text: &TextDisplay, class: TextClass) {
-        let pos = pos + self.offset;
+        let pos = pos;
         let col = self.cols.text_class(class);
-        self.shared
-            .draw_text(&mut self.draw, self.pass, pos.into(), text, col);
+        self.draw.text(pos.into(), text, col);
     }
 
     fn text_effects(&mut self, pos: Coord, text: &dyn TextApi, class: TextClass) {
-        self.shared.draw_text_col_effects(
-            &mut self.draw,
-            self.pass,
-            (pos + self.offset).into(),
+        self.draw.text_col_effects(
+            (pos).into(),
             text.display(),
             self.cols.text_class(class),
             text.effect_tokens(),
@@ -350,21 +335,13 @@ where
     }
 
     fn text_accel(&mut self, pos: Coord, text: &Text<AccelString>, state: bool, class: TextClass) {
-        let pos = Vec2::from(pos + self.offset);
+        let pos = Vec2::from(pos);
         let col = self.cols.text_class(class);
         if state {
             let effects = text.text().effect_tokens();
-            self.shared.draw_text_col_effects(
-                &mut self.draw,
-                self.pass,
-                pos,
-                text.as_ref(),
-                col,
-                effects,
-            );
+            self.draw.text_col_effects(pos, text.as_ref(), col, effects);
         } else {
-            self.shared
-                .draw_text(&mut self.draw, self.pass, pos, text.as_ref(), col);
+            self.draw.text(pos, text.as_ref(), col);
         }
     }
 
@@ -375,7 +352,7 @@ where
         range: Range<usize>,
         class: TextClass,
     ) {
-        let pos = Vec2::from(pos + self.offset);
+        let pos = Vec2::from(pos);
         let col = self.cols.text_class(class);
 
         // Draw background:
@@ -383,7 +360,7 @@ where
             let p1 = Vec2::from(*p1);
             let p2 = Vec2::from(*p2);
             let quad = Quad::with_coords(pos + p1, pos + p2);
-            self.draw.rect(self.pass, quad, self.cols.text_sel_bg);
+            self.draw.rect(quad, self.cols.text_sel_bg);
         }
 
         let effects = [
@@ -403,13 +380,12 @@ where
                 aux: col,
             },
         ];
-        self.shared
-            .draw_text_effects(&mut self.draw, self.pass, pos, text, &effects);
+        self.draw.text_effects(pos, text, &effects);
     }
 
     fn edit_marker(&mut self, pos: Coord, text: &TextDisplay, class: TextClass, byte: usize) {
         let width = self.window.dims.font_marker_width;
-        let pos = Vec2::from(pos + self.offset);
+        let pos = Vec2::from(pos);
 
         let mut col = self.cols.text_class(class);
         for cursor in text.text_glyph_pos(byte).rev() {
@@ -419,7 +395,7 @@ where
             p2.1 -= cursor.descent;
             p2.0 += width;
             let quad = Quad::with_coords(p1, p2);
-            self.draw.rect(self.pass, quad, col);
+            self.draw.rect(quad, col);
 
             if cursor.embedding_level() > 0 {
                 // Add a hat to indicate directionality.
@@ -429,52 +405,51 @@ where
                 } else {
                     Quad::with_coords(Vec2(p1.0 - width, p1.1), Vec2(p1.0, p1.1 + height))
                 };
-                self.draw.rect(self.pass, quad, col);
+                self.draw.rect(quad, col);
             }
             // hack to make secondary marker grey:
-            col = self.cols.button_disabled;
+            col = col.average();
         }
     }
 
     fn menu_entry(&mut self, rect: Rect, state: InputState) {
         if let Some(col) = self.cols.menu_entry(state) {
-            let quad = Quad::from(rect + self.offset);
-            self.draw.rect(self.pass, quad, col);
+            let quad = Quad::from(rect);
+            self.draw.rect(quad, col);
         }
     }
 
-    fn button(&mut self, rect: Rect, state: InputState) {
-        let outer = Quad::from(rect + self.offset);
-        let col = self.cols.button_state(state);
+    fn button(&mut self, rect: Rect, col: Option<color::Rgb>, state: InputState) {
+        let outer = Quad::from(rect);
+        let col = col.map(|c| c.into()).unwrap_or(self.cols.button);
+        let col = ColorsLinear::adjust_for_state(col, state);
 
         let inner = outer.shrink(self.window.dims.button_frame as f32);
-        self.draw.rounded_frame(self.pass, outer, inner, 0.0, col);
-        self.draw.rect(self.pass, inner, col);
+        self.draw.rounded_frame(outer, inner, 0.0, col);
+        self.draw.rect(inner, col);
 
         if let Some(col) = self.cols.nav_region(state) {
             let outer = outer.shrink(self.window.dims.inner_margin as f32);
-            self.draw.rounded_frame(self.pass, outer, inner, 0.6, col);
+            self.draw.rounded_frame(outer, inner, 0.6, col);
         }
     }
 
     fn edit_box(&mut self, rect: Rect, state: InputState) {
         let bg_col = self.cols.bg_col(state);
-        self.draw_edit_box(rect + self.offset, bg_col, self.cols.nav_region(state));
+        self.draw_edit_box(rect, bg_col, self.cols.nav_region(state));
     }
 
     fn checkbox(&mut self, rect: Rect, checked: bool, state: InputState) {
         let bg_col = self.cols.bg_col(state);
         let nav_col = self.cols.nav_region(state).or(Some(bg_col));
 
-        let inner = self.draw_edit_box(rect + self.offset, bg_col, nav_col);
+        let inner = self.draw_edit_box(rect, bg_col, nav_col);
 
         if let Some(col) = self.cols.check_mark_state(state, checked) {
             let radius = inner.size().sum() * (1.0 / 16.0);
             let inner = inner.shrink(self.window.dims.inner_margin as f32 + radius);
-            self.draw
-                .rounded_line(self.pass, inner.a, inner.b, radius, col);
-            self.draw
-                .rounded_line(self.pass, inner.ab(), inner.ba(), radius, col);
+            self.draw.rounded_line(inner.a, inner.b, radius, col);
+            self.draw.rounded_line(inner.ab(), inner.ba(), radius, col);
         }
     }
 
@@ -482,20 +457,20 @@ where
         let bg_col = self.cols.bg_col(state);
         let nav_col = self.cols.nav_region(state).or(Some(bg_col));
 
-        let inner = self.draw_edit_box(rect + self.offset, bg_col, nav_col);
+        let inner = self.draw_edit_box(rect, bg_col, nav_col);
 
         if let Some(col) = self.cols.check_mark_state(state, checked) {
             let inner = inner.shrink(self.window.dims.inner_margin as f32);
-            self.draw.circle(self.pass, inner, 0.5, col);
+            self.draw.circle(inner, 0.5, col);
         }
     }
 
     fn scrollbar(&mut self, rect: Rect, h_rect: Rect, _dir: Direction, state: InputState) {
         // track
-        let outer = Quad::from(rect + self.offset);
+        let outer = Quad::from(rect);
         let inner = outer.shrink(outer.size().min_comp() / 2.0);
         let col = self.cols.frame;
-        self.draw.rounded_frame(self.pass, outer, inner, 0.0, col);
+        self.draw.rounded_frame(outer, inner, 0.0, col);
 
         // handle
         self.draw_handle(h_rect, state);
@@ -503,25 +478,24 @@ where
 
     fn slider(&mut self, rect: Rect, h_rect: Rect, dir: Direction, state: InputState) {
         // track
-        let mut outer = Quad::from(rect + self.offset);
+        let mut outer = Quad::from(rect);
         outer = match dir.is_horizontal() {
             true => outer.shrink_vec(Vec2(0.0, outer.size().1 * (3.0 / 8.0))),
             false => outer.shrink_vec(Vec2(outer.size().0 * (3.0 / 8.0), 0.0)),
         };
         let inner = outer.shrink(outer.size().min_comp() / 2.0);
         let col = self.cols.frame;
-        self.draw.rounded_frame(self.pass, outer, inner, 0.0, col);
+        self.draw.rounded_frame(outer, inner, 0.0, col);
 
         // handle
         self.draw_handle(h_rect, state);
     }
 
     fn progress_bar(&mut self, rect: Rect, dir: Direction, _: InputState, value: f32) {
-        let outer = Quad::from(rect + self.offset);
+        let outer = Quad::from(rect);
         let mut inner = outer.shrink(self.window.dims.frame as f32);
         let col = self.cols.frame;
-        self.draw
-            .rounded_frame(self.pass, outer, inner, BG_SHRINK_FACTOR, col);
+        self.draw.rounded_frame(outer, inner, BG_SHRINK_FACTOR, col);
 
         if dir.is_horizontal() {
             inner.b.0 = inner.a.0 + value * (inner.b.0 - inner.a.0);
@@ -529,11 +503,11 @@ where
             inner.b.1 = inner.a.1 + value * (inner.b.1 - inner.a.1);
         }
         let col = self.cols.button;
-        self.draw.rect(self.pass, inner, col);
+        self.draw.rect(inner, col);
     }
 
     fn image(&mut self, id: ImageId, rect: Rect) {
-        let rect = Quad::from(rect + self.offset);
-        self.shared.draw_image(&mut self.draw, self.pass, id, rect);
+        let rect = Quad::from(rect);
+        self.draw.image(id, rect);
     }
 }
